/**
 * clearFusionCMS script to manage sidebar position.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS
 * @copyright Copyright (c) 2015 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearFusionCMS.com
 */

$(document).ready(function() {
	var sidebar = $('#sidebar'),
		sidebarContent = $('#sidebar .content'),
		sidebarTree = $('#sidebar .treeviewwidgetContainer'),
		sidebarHeader = $('#sidebar .header'),
		sideContainer = $('#sideContainer'),
		sidebarCard = $('#sidebar .card'),
		appbar = $('.appbar'),
		submitList = $('#submit-list');
		win = $(window);

	function updateSidebar() {
		try {
			if(win.width() > 767) {

				var appbarHgt = appbar.height() +
						parseInt(appbar.css('padding-top')) +
						parseInt(appbar.css('padding-bottom')),
					toolbarHgt,
					submitListHgt,
					offset = Math.max(appbarHgt - sidebarCard[0].offsetTop, sideContainer[0].offsetTop - win.scrollTop());

				if(sidebarHeader.length) {
					toolbarHgt = sidebarHeader.height() +
							parseInt(sidebarHeader.css('padding-top')) +
							parseInt(sidebarHeader.css('padding-bottom'));
				}
				else
					toolbarHgt = 0;

				if(submitList.length && submitList.css('position') == 'fixed') {
					submitListHgt = submitList.height() +
						parseInt(submitList.css('padding-top')) +
						parseInt(submitList.css('padding-bottom'));
				}
				else
					submitListHgt = 0;

				// Set position of sidebar
				sidebar
					.addClass('sidebarFixed')
					.css('left', sideContainer[0].offsetLeft + parseInt(sideContainer.css('padding-left')) + 'px')
					.css('width', sideContainer.width() + 'px')
					.css('top', offset + 'px');

				sidebarContent.css('height', (win.height() - toolbarHgt - submitListHgt - offset - sidebarCard[0].offsetTop) + 'px');
				sidebarTree.css('height', (win.height() - toolbarHgt - submitListHgt - offset - sidebarCard[0].offsetTop) + 'px');
			}
			else {
				sidebar
					.removeClass('sidebarFixed')
					.css('top', 'inherit')
					.css('left', 'auto')
					.css('width', 'auto');
				sidebarContent.css('height', 'auto');
				sidebarTree.css('height', 'auto');
			}
		} catch(e) {
		}
	}

	win.on('scroll', updateSidebar)
		.on('resize', updateSidebar)
		.on('load', updateSidebar);
	updateSidebar();

	// Main menu
	$('#mainMenuLevel2 ul').attr('aria-hidden', 'true');

	$('#mainMenu li').on('click', function(e) {
		var mnuName = $(this).attr('data-mainmenu');

		if(!mnuName) {
			$('#mainMenuLevel2').removeClass('active');
			$('#mainMenuOverlay').removeClass('active');
			$('#mainMenu li').removeClass('active');
		}
		else {
			var mnu = $('#' + mnuName);

			if($('#mainMenuLevel2').hasClass('active') && mnu.hasClass('active')) {
				$('#mainMenuLevel2').removeClass('active');
				$('#mainMenuOverlay').removeClass('active');
				$(this).removeClass('active');
			}
			else {
				$('#mainMenuOverlay').addClass('active');
				$('#mainMenuLevel2').addClass('active');
				$('#mainMenuLevel2 ul').removeClass('active').attr('aria-hidden', 'true');
				mnu.addClass('active').attr('aria-hidden', 'false');
				$('#mainMenu li').removeClass('active');
				$(this).addClass('active');
			}
		}
	});

	$('#mainMenuOverlay').on('click', function() {
		$('#mainMenuLevel2').removeClass('active');
		$('#mainMenu li').removeClass('active');
		$('#mainMenuOverlay').removeClass('active');

		$('#mainMenu').removeClass('exposed');
		$('.page-wrapper').removeClass('exposed');
		$('#mainMenuExpose').attr('data-menustate', false);
	});

	// Expose button
	$('#mainMenuExpose').on('click', function() {
		if($(this).attr('data-menustate')) {
			$(this).attr('data-menustate', false);
			$('#mainMenu').removeClass('exposed');
			$('.page-wrapper').removeClass('exposed');
		}
		else {
			$(this).attr('data-menustate', true);
			$('#mainMenu').addClass('exposed');
			$('.page-wrapper').addClass('exposed');
			$('#mainMenuOverlay').addClass('active');
		}

		setTimeout(updateSidebar, 180);
	});

	// Tooltips
	$('#mainMenu li').each(function() {
		$(this).attr('data-menutitle', $(this).attr('title'));
		$(this).attr('title', '');
	});

	$('#mainMenu li').on('mouseenter', function() {
		if($(window).width() > 767) {
			$('#mainMenuTip')
					.css('display', 'block')
					.css('top', $(this).offset().top + 'px')
					.html($(this).attr('data-menutitle'));
		}
	}).on('mouseleave', function() {
		$('#mainMenuTip').css('display', 'none');
	});

	// Add working indicator to form submit
	$('form').on('submit', function(e) {
		$('#submit-list input').addClass('disabled');
		$('#submit-list button').addClass('disabled');
		$.toastHide();
		$('.alertfade').addClass('hide');
	}).on('formValidationFailed', function(e) {
		$('#submit-list input').removeClass('disabled');
		$('#submit-list button').removeClass('disabled');
	});

	// Update the notices areas
	var msgCount = 0;
	if(typeof managerPath !== 'undefined') {
		var source = null;

		function updateNotices(d) {
			if (d.hasNotices) {
				msgCount = d.noticeCount;
				$('#notifyCount').removeClass('hide').html(d.noticeCount);

				var html = '';
				for(var i=0;i< d.notices.length;i++) {
					html += '<li><div class="msg">';

					if(d.notices[i].type == 1) {
						html += '<i class="material-icons info">info_outline</i>';
					}
					else if(d.notices[i].type == 2) {
						html += '<i class="material-icons warning">warning</i>';
					}
					else if(d.notices[i].type == 3) {
						html += '<i class="material-icons error">error_outline</i>';
					}

					if(d.notices[i].count !== null)
						html += '<b>' + d.notices[i].count + '</b>';

					html += d.notices[i].text;

					if(d.notices[i].link !== null) {
						html = '<a href="' + d.notices[i].link + '">' + html + '</a>';
					}

					html += '</div></li>';
				}
				$('#cfNotifications').html(html);
			}
			else {
				msgCount = 0;
				$('#notifyCount').addClass('hide');
				$('#cfNotifications').html('');
			}
		}

		function getNotices() {
			// If SSE object then close it as it's not responded
			if(source) {
				source.close();
				source = null;
			}

			$.get(managerPath + 'getnotices/', updateNotices);
		}

		// Setup interval for notice updates and force a load
		var noticeTimer = setInterval(getNotices, 5000);
		getNotices();

		// Open Server-Sent Event connection
		if(window.EventSource) {
			source = new EventSource(managerPath + 'getnotices/');
			source.addEventListener('message', function(e) {
				// Kill the notice update timer as SSE working
				if(noticeTimer) {
					clearInterval(noticeTimer);
					noticeTimer = false;
				}

				// Update the user
				updateNotices(JSON.parse(e.data));
			});
		}
	}

	// Alert menu display handler
	$('.notify-icon .fa-bell').on('click', function(e) {
		if($('#cfNotifications').hasClass('closed') && msgCount) {
			$('#cfNotifications').removeClass('closed').attr('aria-hidden', 'false');
		}
		else {
			$('#cfNotifications').addClass('closed').attr('aria-hidden', 'true');
		}
		e.preventDefault();
		return false;
	});

	$('body').on('click', function(e) {
		if(!$(e.target).parent().hasClass('notify-icon')) {
			$('#cfNotifications').addClass('closed').attr('aria-hidden', 'true');
		}
	});
});

(function() {
	// Update focus to force scroll
	var extend = window.jQuery || window.fusionLib;
	var fnFocus = extend.fn.focus;
	extend.fn.focus = function() {
		fnFocus.call(this);

		// Find the label container
		var e = extend(this[0]),
			l = extend('#' + e.attr('id') + '-lblcontainer');

		if(l.length) {
			var top = l.offset().top;
			if(top <= extend(window).scrollTop() || top > (extend(window).scrollTop() + (extend(window).height() * .5))) {
				extend(window).scrollTop(top - extend('.appbar').height() - 20);
			}
		}

		return this;
	};

})();
