<?php

/**
 * Application configuration file.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS
 * @copyright Copyright (c) 2012 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * Base Path
 *
 * basepath - The URI excluding the domain name to the installation, e.g. if not using mod_rewrite
 * and installed at http://www.example.com/index.php then the base URI will be
 * '/index.php'. The same address using mod_rewrite would be http://www.example.com/
 */
$config['basepath'] = '/';
//$config['basepath'] = '/index.php';

/**
 * Database
 *
 * db - The connection information for the database.
 * dbslave - Optional slave database connection information.
 */
$config['db'] = 'mysqli://user:password@dbserver/database';

/**
 * The path to the manager, defaults to manager
 */
//$config['managerPath'] = 'manager';

/**
 * Code Buffer
 *
 * usejQuery - true if code buffers should output jQuery; else fusionLib JavaScript is used.
 * managerUsejQuery - true if code buffers should output jQuery within the manager interface.
 */
$config['usejQuery'] = true;
//$config['managerUsejQuery'] = true;

/**
 * Throttle Requests
 *
 * throttleTime - The time over which requests are monitored in seconds.
 * throttleLimit - The limit of requests in throttleTime seconds.
 *
 * Both values must be set to throttle requests.
 */
//$config['throttleTime'] = 60;
//$config['throttleLimit'] = 100;

/**
 * Cache
 *
 * If nothing is specified then file caching will be used.
 *
 * cacheDriver - The cache driver, one of File, APC, Sqlite, Memcached, Memcache, Redis or Dummy. If not given then File will be used.
 * cacheLifetime - The default cache lifetime.
 * cachePrefix - Optional prefix for cache keys.
 * cacheServer - memcache, an array of server addresses and port pairs, localhost,11211
 *             - redis, a string password@localhost:port/db, only the server name is required.
 */
//$config['cacheDriver'] = 'File';
//$config['cacheLifetime'] = 3660;
//$config['cachePrefix'] = __FILE__;
//$config['cacheServer'] = 'password@ip:port/db';

/**
 * Session
 *
 * sessionDriver - The session driver, one of Db, File, Memcached, Memcache or Redis. If not given then Db will be used.
 * sessionLife - The lifetime of a session in seconds, null to use the system default.
 * sessionCookieSession - true to use a session cookies.
 * sessionCookieName - Name to use for session cookies.
 * sessionCookieDomain - Domain the cookie is valid for.
 * sessionCookiePath - Path the cookie is valid for.
 * sessionServer - memcache, an array of server addresses and port pairs, localhost,11211
 *               - redis, a string password@localhost:port/db, only the server name is required.
 */
$config['sessionDriver'] = 'Db';
$config['sessionLife'] = 172800;
//$config['sessionCookieSession'] = true;
//$config['sessionCookieName'] = '__sessionid';
//$config['sessionCookieDomain'] = '';
//$config['sessionCookiePath'] = '/';
//$config['sessionServer'] = 'password@ip:port/db';

/**
 * Authentication
 *
 * authCookieName - Name to use for authentication cookie.
 * authCookieDomain - Domain the cookie is valid for.
 * authCookiePath - Path the cookie is valid for.
 * authCookieMgrName - Name to use for the manager authentication cookie.
 */
//$config['authCookieName'] = '__userid';
//$config['authCookieDomain'] = '';
//$config['authCookiePath'] = '/';
//$config['authCookieMgrName'] = '__managerauth'

/**
 * Time in seconds clearFusionCMS should cache public pages for.
 */
$config['pageCacheTime'] = 86400;

/**
 * The umask, normally 0002 for suPHP etc and 0000 for mod_php
 */
$config['umask'] = 0002;

/**
 * Authenticate against a master install of clearFusionCMS
 *
 * userAdapter - The adapter to use
 * userAdapterEndpoint - The URI of the API on the master, example.com/manager/api/
 * userAdapterAPIKey - The API key from the master
 * userAdapterCacheTime - The time in seconds to cache user information
 */
//$config['userAdapter'] = 'flUserAdapterRemote';
//$config['userAdapterEndpoint'] = '';
//$config['userAdapterAPIKey'] = '';
//$config['userAdapterCacheTime'] = 60;

/**************************
 * Do Not Edit Below Here
 *************************/

/**
 * Debug
 *
 * Enable the use of the development version or JavaScript, not the minified version.
 */
//flBufferCode::useDevJS();

/**
 * Enable full debug output on crash, must not be used on production systems.
 */
//flExceptionReporter::showFullDebug();
