<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\campaignmonitor
 * @copyright Copyright (c) 2013 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * campaignmonitorSettings Class
 *
 * Class to implement settings for Campaign Monitor.
 *
 * @package clearFusionCMS\module\campaignmonitor
 */
class campaignmonitorSettings implements cfModuleSettingsInterface {

	/**
	 * Get a list of settings groups.
	 *
	 * @return array Array of setting groups, name => func name
	 */
	function getSettingsGroups() {
		if(!clearFusionCMS::getUser()->hasPermission('cfCampaignMonitor.settings'))
			return array();

		return array(
			__('API Key', 'campaignmonitorBackend') => 'apikey'
		);
	}

	/**
	 * Implement API Key settings.
	 *
	 * @param flForm $form The form object to use.
	 * @param flDb $db The database object.
	 * @return bool true if the settings were updated, false if failed to update or null if no change.
	 */
	function apikeySetting($form, $db) {
		if(!clearFusionCMS::getUser()->hasPermission('cfCampaignMonitor.settings'))
			return null;

		$cfg = clearFusionCMS::getSiteConfig('cfCampaignMonitor');

		$apiKey = new flFormElementText($form, 'apikey', $cfg->apiKey);
		$apiKey->setLabel(__('API Key', 'campaignmonitorBackend'))
			->setRequired()
			->addValidator(new flFormValidatorRequired())
			->setDescription(__('You can find API keys in the Campaign Monitor admin under Client Settings, Edit, Show client API Info.', 'campaignmonitorBackend'));

		$clientId = new flFormElementText($form, 'clientid', $cfg->clientId);
		$clientId->setLabel(__('Client ID', 'campaignmonitorBackend'))
			->setDescription(__('If not given the list of available lists will not be populated below.', 'campaignmonitorBackend'));

		$avail = new flFormElementPane($form, 'lists', __('Not Available', 'campaignmonitorBackend'));

		$submit = new flFormElementSubmit($form, 'update');
		$submit->setCaption(__('Update Settings', 'campaignmonitorBackend'))->setCSS('primary');

		$form->groupElements('keys', array($apiKey, $clientId))
			->groupElements('lists', array($avail))
			->addTab(__('API Key', 'campaignmonitorBackend'), 'keys')
			->addTab(__('Lists', 'campaignmonitorBackend'), 'lists')
			->groupElements('submit', array($submit), new flFormDecoratorList(new flFormDecoratorDl()));

		$form->focusElement($apiKey);

		$r = null;
		if($form->isSubmitted() && $form->isValid()) {
			cfSettingsAudit::update(
				__('Campaign Monitor', 'campaignmonitorBackend'),
				__('API Key', 'campaignmonitorBackend'),
				[
					'apiKey' => $cfg->get('apiKey'),
					'clientId' => $cfg->get('clientId')
				]
			);

			$cfg->apiKey = $apiKey->getValue();
			$cfg->clientId = $clientId->getValue();
			$r = true;
		}

		// If has an api key then list the available lists
		if(!empty($cfg->apiKey) && !empty($cfg->clientId)) {
			$mc = new campaignmonitor($cfg->apiKey, $cfg->clientId);
			if($lists = $mc->getLists()) {
				$table = new flWidgetTable();
				$table->setClass('tablewidget bordered striped rowhover')
					->setHeadings(__('ID', 'campaignmonitorBackend'), __('Name', 'campaignmonitorBackend'))
					->setColData('id', 'name');

				foreach($lists as $l) {
					$table->addRow(array(
						'id' => $l->ListID,
						'name' => $l->Name
					));
				}

				$avail->setContent($table->render() . clearFusionCMS::getController()->alertMessage('Info', __('To create a subscribe form use &#91;&#91;campaignmonitor.subscribe &amp;listId=listid&#93;&#93;<br />To create an unsubscribe form use &#91;&#91;campaignmonitor.unsubscribe &amp;listId=listid&#93;&#93;<br />Where listid is one of the above list IDs.', 'campaignmonitorBackend')));
			}
		}

		return $r;
	}
}
