<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\ckeditor
 * @copyright Copyright (c) 2012 - 2016 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * ckeditorWYSIWYG Class
 *
 * Class to draw the editor.
 *
 * Includes the additional plugins:
 * - Styles Parser
 * - Word Count & Char Count Plugin
 * - Youtube Plugin
 * - Insert Smiley
 *
 * @package clearFusionCMS\module\ckeditor
 */
class ckeditorWYSIWYG extends flFormElementWYSIWYGCodeModule {
	/**
	 * Get the CSS class to add to the textarea.
	 *
	 * @return string The CSS class or null if not used.
	 */
	function getCSSClass() {
		return null;
	}

	/**
	 * Get if dimensions must be included in the style of the textarea.
	 *
	 * @return bool true if dimensions in style attribute; else false.
	 */
	function dimensionsInStyle() {
		return false;
	}

	/**
	 * Render the editor code.
	 *
	 * @param flDb $db The database object.
	 * @param flBufferCode $codeBuffer The code buffer.
	 * @param array $settings The array of settings for editor.
	 * @return string The HTML for inclusion.
	 */
	function render($db, $codeBuffer, $settings) {

		$codeBuffer->bufferBodyJS('system/modules/ckeditor/ckeditor/ckeditor.js');

		$js = "CKEDITOR.disableAutoInline = true;
	CKEDITOR.replace('{$this->name}', {
	toolbar : [
		{ name: 'clipboard', items : [ 'Cut','Copy','Paste','PasteText','PasteFromWord','-','Undo','Redo' ] },
		{ name: 'basicstyles', items : [ 'Bold','Italic','Strike','TextColor','BGColor','-','RemoveFormat' ] },
		{ name: 'links', items : [ 'Link','Unlink','Anchor' ] },
		{ name: 'tools', items : [ 'Replace','Source','Maximize' ] },
		'/',
		{ name: 'styles', items : [ " . (!empty($settings['css']) ? "'Styles', " : '') . "'Format','Font','FontSize' ] },
		{ name: 'paragraph', items : [ 'NumberedList','BulletedList','-','Outdent','Indent','-','Blockquote','-','JustifyLeft','JustifyCenter','JustifyRight' ] },
		{ name: 'insert', items : [ 'Image', 'Youtube','Table','HorizontalRule','Smiley','SpecialChar' ] }
	],
	youtube_related : false,
	wordcount : { showCharCount : true }";

		if(!empty($settings['fileBrowser']))
			$js .= ",
	filebrowserBrowseUrl : '$settings[fileBrowser]ckeditor/',
	filebrowserWindowWidth : '620',
	filebrowserWindowHeight : '500',
	baseHref : '" . clearFusionCMS::getRequest()->getBasePath(true) . "'";

		if(!empty($settings['css']))
			$js .= ",
	extraPlugins : 'stylesheetparser,youtube,wordcount',
	contentsCss : '$settings[css]',
	stylesSet : []";
		else
			$js .= ",
	extraPlugins : 'youtube,wordcount'";

		// Load locale if available
		$locale = strtolower(fusionLib::getLocale(true));
		$localeShort = explode('-', $locale);
		$localeShort = $localeShort[0];
		if(is_readable(FLMODPATH . "ckeditor/ckeditor/lang/$locale.js"))
			$js .= ",\n\tlanguage: '$locale'";
		else if(is_readable(FLMODPATH . "ckeditor/ckeditor/lang/$localeShort.js"))
			$js .= ",\n\tlanguage: '$localeShort'";

		$js .= "
});";

		$codeBuffer->bufferBodyJS($js);

		return '';
	}
}
