<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\cloudflare
 * @copyright Copyright (c) 2014 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * cloudflareBackendController Class
 *
 * Backend controller for managing cloudflare.
 *
 * @package clearFusionCMS\module\cloudflare
 */
class cloudflareBackendController extends cfBackendController {

	/**
	 * Blog dashboard action.
	 */
	function dashboardAction() {
		$this->testPermission('cfCloudFlare.access');
		$this->breadcrumbTrail->addCrumb(__('Cloudflare', 'cloudflareBackend'), $this->assemble('cloudflare', 'dashboard'));

		$v = new flVariables('cfCloudFlare', clearFusionCMS::getDb());
		$site = clearFusionCMS::getSite();

		$content = '';
		$msg = $this->session->flashGet('cfManagerOpSuccess') ? $this->session->flashGet('cfManagerOpSuccess') : '';

		// Get the domain name we're working with
		$domainName = $this->db->fetchOne('SELECT domain FROM cf_sites WHERE shortname=? && redirectto=\'\'', $site);

		// Query CloudFlare for current domain settings
		$cfg = new flConfigDb('cfCloudFlare', $this->db);
		if(empty($cfg->email) || empty($cfg->tkn)) {
			$msg = $this->alertMessage('Error', __('You must configure your Cloudflare API credentials via Settings.', 'cloudflareBackend'));
		}
		else {
			$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones?name=' . urlencode(preg_replace('/^www\./', '', $domainName)));
			$client->addHeader('X-Auth-Email', $cfg->email)
				->addHeader('X-Auth-Key', $cfg->tkn)
				->addHeader('Content-Type', 'application/json');

			$data = null;
			if ($client->sendRequest('GET') && $client->getResponseCode() == 200) {
				$data = json_decode($client->getBody());
				if (!$data->success || !count($data->result)) {
					$msg = $this->alertMessage('Error', count($data->messages) ? $data->messages[0] : __('The domain does not appear to be managed by Cloudflare.', 'cloudflareBackend'));
					$data = null;
				}
			}

			if ($data) {
				$form = new flForm($this->request, 'contentform', $this->assemble('cloudflare', 'dashboard'), true);

				$devSubmit = new flFormElementSubmit($form, 'devsubmit');

				if ($data->result[0]->development_mode <= 0) {
					$devSubmit->setCaption(__('Enable Development Mode', 'cloudflareBackend'))->setCSS('accent')->setDescription($this->alertMessage('Info', __('Development Mode allows you to temporarily suspend Cloudflare\'s edge caching and minification features.', 'cloudflareBackend')));
					unset($v->$site);
				} else {
					$expire = new flDate('@' . ($t = time() + $data->result[0]->development_mode));
					$expire->setTimezone($this->user->getTimezone());
					$devSubmit->setCaption(__('Disable Development Mode', 'cloudflareBackend'))->setCSS('primary')->setDescription($this->alertMessage('Info', sprintf(__('Development mode will automatically end at %s', 'cloudflareBackend'), $expire->toString())));
					$v->$site = $t;
				}

				$purgeSubmit = new flFormElementSubmit($form, 'purgesubmit');
				$purgeSubmit->setCaption(__('Purge Cloudflare Cache', 'cloudflareBackend'))->setCSS('danger')->setDescription($this->alertMessage('Warning', __('The cache may take up to 48 hours to rebuild after purging it.', 'cloudflareBackend')));

				$url = new flFormElementUri($form, 'url');
				$url->setLabel(__('Purge URL', 'cloudflareBackend'))->setDescription(__('The full URL of purge from Cloudflare.', 'cloudflareBackend'));

				$purgeFileSubmit = new flFormElementSubmit($form, 'purgeurl');
				$purgeFileSubmit->setCaption(__('Purge URL From Cloudflare', 'cloudflareBackend'))->setCSS('primary');

				$form->groupElements('content1', array($devSubmit, $purgeSubmit))->groupElements('content2', array($url, $purgeFileSubmit));

				if ($form->isSubmitted() && $form->isValid()) {

					// Disabling/enabling dev mode
					if ($devSubmit->getValue()) {
						$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones/' . urlencode($data->result[0]->id) . '/settings/development_mode');
						$client->addHeader('X-Auth-Email', $cfg->email)
							->addHeader('X-Auth-Key', $cfg->tkn)
							->addHeader('Content-Type', 'application/json')
							->setContentType('application/json')
							->setPostData(json_encode(array('value' => $data->result[0]->development_mode > 0 ? 'off' : 'on')));

						if ($client->sendRequest('PATCH') && $client->getResponseCode() == 200) {
							if ($data->result[0]->development_mode) {
								$this->session->flashSet('cfManagerOpSuccess', $this->alertMessage('Success', __('The request to disable development mode has been completed.', 'cloudflareBackend')));
								cloudflareAudit::enaledDevMode();
							}
							else {
								$this->session->flashSet('cfManagerOpSuccess', $this->alertMessage('Success', __('The request to enable development mode has been completed.', 'cloudflareBackend')));
								cloudflareAudit::enaledDevMode();
							}
							$this->response->redirectTo($this->assemble('cloudflare', 'dashboard', array(), true));
						} else
							$msg = $this->alertMessage('Error', __('Failed to change the state of development mode.', 'cloudflareBackend'));
					}

					if ($purgeSubmit->getValue()) {
						$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones/' . urlencode($data->result[0]->id) . '/purge_cache');
						$client->addHeader('X-Auth-Email', $cfg->email)
							->addHeader('X-Auth-Key', $cfg->tkn)
							->addHeader('Content-Type', 'application/json')
							->setContentType('application/json')
							->setPostData(json_encode(array('purge_everything' => true)));

						if ($client->sendRequest('DELETE') && $client->getResponseCode() == 200) {
							cloudflareAudit::purgeAll();
							$this->session->flashSet('cfManagerOpSuccess', $this->alertMessage('Success', __('Cache purge request has been completed.', 'cloudflareBackend')));
							$this->response->redirectTo($this->assemble('cloudflare', 'dashboard', array(), true));
						} else
							$msg = $this->alertMessage('Error', __('Failed to purge the Cloudflare cache.', 'cloudflareBackend'));
					}

					if ($purgeFileSubmit->getValue() && $url->getValue()) {
						$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones/' . urlencode($data->result[0]->id) . '/purge_cache');
						$client->addHeader('X-Auth-Email', $cfg->email)
							->addHeader('X-Auth-Key', $cfg->tkn)
							->addHeader('Content-Type', 'application/json')
							->setContentType('application/json')
							->setPostData(json_encode(array('files' => array($url->getValue()))));

						if ($client->sendRequest('DELETE') && $client->getResponseCode() == 200) {
							cloudflareAudit::purgeURL($url->getValue());
							$this->session->flashSet('cfManagerOpSuccess', $this->alertMessage('Success', __('URL purge request has been completed.', 'cloudflareBackend')));
							$this->response->redirectTo($this->assemble('cloudflare', 'dashboard', array(), true));
						} else
							$msg = $this->alertMessage('Error', __('Failed to purge the Cloudflare cache.', 'cloudflareBackend'));
					}
				}

				$content .= $form->render($this->tpl->getCodeBuffer());
			}
		}

		$this->renderPage(__('Cloudflare', 'cloudflareBackend'), $this->renderBlock(__('Cloudflare', 'cloudflareBackend'), $content, '', $msg));
	}
}
