<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\cloudflare
 * @copyright Copyright (c) 2014 - 2015 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * cloudflareEvents Class
 *
 * Class to process CloudFlare module events.
 *
 * @package clearFusionCMS\module\cloudflare
 */
class cloudflareEvents {

	/**
	 * Event handler for system notify updates.
	 *
	 * @param flEvent $evt The event object.
	 */
	function onCacheFlush($evt) {
		// Get the domain name we're working with
		$domainName = clearFusionCMS::getDb()->fetchOne('SELECT domain FROM cf_sites WHERE shortname=? && redirectto=\'\'', clearFusionCMS::getSite());

		$cfg = new flConfigDb('cfCloudFlare', clearFusionCMS::getDb());

		if(!empty($cfg->email) || !empty($cfg->tkn)) {

			// Look up the ID of the zone
			$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones?name=' . urlencode(preg_replace('/^www\./', '', $domainName)));
			$client->addHeader('X-Auth-Email', $cfg->email)
				->addHeader('X-Auth-Key', $cfg->tkn)
				->addHeader('Content-Type', 'application/json');

			$data = null;
			if ($client->sendRequest('GET') && $client->getResponseCode() == 200) {
				$data = json_decode($client->getBody());
				if (!$data->success || !count($data->result)) {
					$data = null;
				}
			}

			if($data) {
				// Clear the cache
				$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones/' . urlencode($data->result[0]->id) . '/purge_cache');
				$client->addHeader('X-Auth-Email', $cfg->email)
					->addHeader('X-Auth-Key', $cfg->tkn)
					->addHeader('Content-Type', 'application/json')
					->setContentType('application/json')
					->setPostData(json_encode(array('purge_everything' => true)))
					->sendRequest('DELETE');
			}
		}
	}

	/**
	 * Event handler to purge a list of files from the cache.
	 *
	 * @param flEvent $evt The event object.
	 */
	function onCachePurge($evt) {
		$cfg = new flConfigDb('cfCloudFlare', clearFusionCMS::getDb());
		if(!empty($cfg->email) || !empty($cfg->tkn)) {
			$files = $evt->getParam('files', array());

			// Get the domain name we're working with
			$domainName = clearFusionCMS::getDb()->fetchOne('SELECT domain FROM cf_sites WHERE shortname=? && redirectto=\'\'', clearFusionCMS::getSite());

			// Look up the ID of the zone
			$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones?name=' . urlencode(preg_replace('/^www\./', '', $domainName)));
			$client->addHeader('X-Auth-Email', $cfg->email)
				->addHeader('X-Auth-Key', $cfg->tkn)
				->addHeader('Content-Type', 'application/json');

			$data = null;
			if ($client->sendRequest('GET') && $client->getResponseCode() == 200) {
				$data = json_decode($client->getBody());
				if (!$data->success || !count($data->result)) {
					$data = null;
				}
			}

			if($data) {
				$http = 'http://' . $domainName;
				$https = 'https://' . $domainName;
				$list = array();

				foreach($files as $f) {
					$list[] = $http . $f;
					$list[] = $https . $f;
				}

				$client = new flHttpClient('https://api.cloudflare.com/client/v4/zones/' . urlencode($data->result[0]->id) . '/purge_cache');
				$client->addHeader('X-Auth-Email', $cfg->email)
					->addHeader('X-Auth-Key', $cfg->tkn)
					->addHeader('Content-Type', 'application/json')
					->setContentType('application/json')
					->setPostData(json_encode(array('files' => $list)))
					->sendRequest('DELETE');
			}
		}
	}

	/**
	 * Event handler for system requesting notices.
	 *
	 * @param cfNoticesEvent $evt The event object.
	 */
	function onGetNotices($evt) {
		$v = new flVariables('cfCloudFlare', clearFusionCMS::getDb());

		// Get the dev mode expiry time
		if($t = $v->get(clearFusionCMS::getSite())) {
			$now = time();
			$expire = new flDate('@' . $t);
			$expire->setTimezone(clearFusionCMS::getUser()->getTimezone());

			// If in the future
			if($t >= $now) {
				$offset = $t - $now;
				if($offset / 3600 > 1)
					$x = ceil($offset / 3600) . 'h';
				else if($offset / 60 > 1)
					$x = ceil($offset / 60) . 'm';
				else
					$x = $offset . 's';

				$evt->addMsg(new cfNoticesEventMsg(
					sprintf(__('CloudFlare development mode will automatically end at %s', 'cloudflareBackend'), $expire->toString()),
					$x, clearFusionCMS::getController()->assemble('cloudflare', 'dashboard'), null, cfNoticesEventMsg::INFO));
			}
			// Else if in the past but less then 5 min
			else if($now < $t + 300) {
				$evt->addMsg(new cfNoticesEventMsg(
					sprintf(__('CloudFlare development mode automatically ended at %s', 'cloudflareBackend'), $expire->toString()),
					null, clearFusionCMS::getController()->assemble('cloudflare', 'dashboard'), null, cfNoticesEventMsg::INFO));
			}
		}
	}
}
