<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\cloudflare
 * @copyright Copyright (c) 2014 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * cloudflareSettings Class
 *
 * Class to implement settings for the CloudFlare module.
 *
 * @package clearFusionCMS\module\cloudflare
 */
class cloudflareSettings implements cfModuleSettingsInterface {

	/**
	 * Get a list of settings groups.
	 *
	 * @return array Array of setting groups, name => func name
	 */
	function getSettingsGroups() {
		if(!clearFusionCMS::getUser()->hasPermission('cfCloudFlare.settings'))
			return array();

		return array(
			__('API Credentials', 'cloudflareBackend') => 'api'
		);
	}

	/**
	 * Implement api credentials settings.
	 *
	 * @param flForm $form The form object to use.
	 * @param flDb $db The database object.
	 * @return bool true if the settings were updated, false if failed to update or null if no change.
	 */
	function apiSetting($form, $db) {
		if(!clearFusionCMS::getUser()->hasPermission('cfCloudFlare.settings'))
			return null;

		$cfg = new flConfigDb('cfCloudFlare', $db);

		$email = new flFormElementEmail($form, 'apiemail', $cfg->get('email'));
		$email->setLabel(__('Email Address', 'cloudflareBackend'))
			->setDescription(__('The email address associated with the API key.', 'cloudflareBackend'))
			->setRequired()
			->addValidator(new flFormValidatorRequired());

		$tkn = new flFormElementText($form, 'apitkn', $cfg->get('tkn'));
		$tkn->setLabel(__('API Key', 'cloudflareBackend'))
			->setDescription(__('The API key from your Cloudflare account page.', 'cloudflareBackend'))
			->setRequired()
			->addValidator(new flFormValidatorRequired())
			->addValidator(new flFormValidatorMaxLength(64));

		$submit = new flFormElementSubmit($form, 'update');
		$submit->setCaption(__('Update Settings', 'cloudflareBackend'))->setCSS('primary');

		$form->focusElement($email);

		if($form->isSubmitted() && $form->isValid()) {
			cfSettingsAudit::update(
				__('Cloudflare', 'cloudflareBackend'),
				__('API Credentials', 'cloudflareBackend'),
				[
					'email' => $cfg->get('email'),
					'tkn' => $cfg->get('tkn')
				]
			);

			$cfg->email = $email->getValue();
			$cfg->tkn = $tkn->getValue();

			return true;
		}

		return null;
	}
}
