<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\codemirror
 * @copyright Copyright (c) 2013 - 2014 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * codemirrorInstall Class
 *
 * Module installer for CodeMirror.
 *
 * @package clearFusionCMS\module\codemirror
 */
class codemirrorInstall implements cfInstallInterface {
	/**
	 * The last error message.
	 *
	 * @var string
	 */
	protected $lastError = '';

	/**
	 * Return the last error.
	 *
	 * @return string The last error string.
	 */
	function getLastError() {
		return $this->lastError;
	}

	/**
	 * Install the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if installed; else false.
	 */
	function install($db) {
		// Editor settings
		$siteMgr = new cfSiteManager($db);
		$sites = $siteMgr->listSites();
		foreach($sites as $site => $name) {
			$cfg = $siteMgr->getSiteCfg('cfCodeMirror', $site);
			$cfg->theme = '';
		}

		// Permissions
		$ac = new flAcMgr($db);
		$ac->createPermissionGroup('cfCodeMirror', __noop('CodeMirror'), 'codemirrorBackend'); // 6000
		$ac->createPermission('cfCodeMirror.settings', __noop('Can edit CodeMirror editor settings.'), 6000, 'codemirrorBackend');

		return true;
	}

	/**
	 * Uninstall the module.
	 *
	 * @param flDb The database object to use.
	 * @return bool true if uninstalled; else false.
	 */
	function uninstall($db) {
		$siteMgr = new cfSiteManager($db);
		$sites = $siteMgr->listSites();
		foreach($sites as $site => $name) {
			$cfg = $siteMgr->getSiteCfg('cfCodeMirror', $site);
			$cfg->deleteNamespace();
		}

		// Remove permissions
		$ac = new flAcMgr($db);
		$ac->deletePermissionGroup('cfCodeMirror');

		return true;
	}

	/**
	 * Upgrade the module.
	 *
	 * @param flDb The database object to use.
	 * @param string $installedVer The installed version.
	 * @return bool true if upgraded; else false.
	 */
	function upgrade($db, $installedVer) {
		if(version_compare($installedVer, '4.7.0') < 0) {
			$ac = new flAcMgr($db);
			$ac->createPermissionGroup('cfCodeMirror', __noop('CodeMirror'), 'codemirrorBackend'); // 6000
			$ac->createPermission('cfCodeMirror.settings', __noop('Can edit CodeMirror editor settings.'), 6000, 'codemirrorBackend');
		}

		return true;
	}

	/**
	 * Create a new site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The short name of the site.
	 * @return bool true if the site was created; else false.
	 */
	function createSite($db, $site) {
		return true;
	}

	/**
	 * Delete a site.
	 *
	 * @param flDb $db The database object.
	 * @param string $site The name of the site to delete.
	 * @return bool true if the site was deleted; else false.
	 */
	function deleteSite($db, $site) {
		return true;
	}
}
