<?php

/**
 * Class file for decoding comments module audit logs.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\comments
 * @copyright Copyright (c) 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * commentsAuditHelper Class
 *
 * Class to provide decoding for comments audit trails.
 *
 * @package clearFusionCMS\module\comments
 */
class commentsAuditHelper extends cfAuditHelper {

	/**
	 * Convert the raw data to a human readable message.
	 *
	 * @param array $data The audit data.
	 * @return string The message string.
	 */
	function getMessage($data) {
		$params = empty($data['params']) ? [] : json_decode($data['params'], true);

		$title = isset($params['title']) ? $params['title'] : '';

		$link = '';
		if($info = clearFusionCMS::getDb()->fetchRow('SELECT id FROM {cf_comments} WHERE id=?', $data['cid'])) {
			$link = ' <a href="' . clearFusionCMS::getController()->assemble('comments', 'edit', [$data['cid']]) . '">' . __('Edit the Comment', 'commentsBackend') . '</a>';
		}

		switch($data['action']) {
			case commentsAudit::CREATE:
				$msg = sprintf(__('Created a comment from <strong>%s</strong>.', 'commentsBackend'), $title) . $link;
				break;
			case commentsAudit::UPDATE:
				$msg = sprintf(__('Modified the comment from <strong>%s</strong>.', 'commentsBackend'), $title) . $link;
				break;
			case commentsAudit::DELETE:
				$msg = sprintf(__('Deleted comment from <strong>%s</strong>.', 'commentsBackend'), $title);
				break;
			default:
				$msg = parent::getMessage($data);
				break;
		}

		return $msg;
	}

	/**
	 * @inheritdoc
	 */
	function hasAdditionalData($data) {
		return !in_array($data['action'], [ commentsAudit::CREATE ]);
	}
}
