<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\hipchat
 * @copyright Copyright (c) 2013 - 2016 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * hipchatEvents Class
 *
 * Class to process HipChat module events.
 *
 * @package clearFusionCMS\module\hipchat
 */
class hipchatEvents {

	/**
	 * Event handler for system notify updates.
	 *
	 * @param flEvent $evt The event object.
	 */
	function onNotify($evt) {
		$cfg = clearFusionCMS::getSiteConfig('cfHipChat');

		if(!empty($cfg->token) && !empty($cfg->room) && ($level = $evt->getParam('level', 1)) >= $cfg->level) {
			$colorMap = array(1 => 'gray', 2 => 'purple', 3 => 'green', 4 => 'yellow', 5 => 'red' );
			$level = min(max($level, 1), 5);

			$title = $evt->getParam('title', __('Website Notification', 'hipchatBackend'));
			$link = $evt->getParam('link', '');
			$message = $evt->getParam('message', '');
			$fields = $evt->getParam('fields', array());
			$mention = trim($evt->getParam('mention', ''));
			if(!empty($mention))
				$message = $mention . ' ' . $message;

			// Build the message
			if(empty($link))
				$msg = '<b>' . fusionLib::htmlChars($title) . '</b>';
			else
				$msg = '<b><a href="' . $link . '">' . fusionLib::htmlChars($title) . '</a></b>';
			$msg .= '<br>' . nl2br(fusionLib::htmlChars($message)) . '<br>';
			foreach($fields as $v) {
				$msg .= '<b>' . fusionLib::htmlChars($v['title']) . '</b><br>' . nl2br(fusionLib::htmlChars($v['value'])) . '<br>';
			}
			$msg .= '<br><i>' . clearFusionCMS::getSiteName() . '</i>';

			$client = new flHttpClient('https://api.hipchat.com/v2/room/' . $cfg->room . '/notification');
			$client->setContentType('application/json')
				->addHeader('Authorization', 'Bearer ' . $cfg->token)
				->setPostData(json_encode(array(
				'color' => 'purple',
				'message_format' => 'html',
				'message' => $msg,
				'notify' => $level >= $cfg->notify ? '1' : '0',
				'color' => $colorMap[$level]
			)));

			$client->sendRequest('POST');
		}
	}
}
