<?php

/**
 * Snippet to display MailChimp subscribe form.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\mailchimp
 * @copyright Copyright (c) 2013 - 2016 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/* @var $tplEngine cfTemplateEngine */
$controller = $tplEngine->controller;

$caption = isset($caption) ? $caption : __('Subscribe', 'mailchimp');

if(isset($listId)) {
	$mc = new mailchimp();

	// Cache field information
	$cacheId = '__cfMailChimp' . clearFusionCMS::getDb()->getPrefix() . "-fields-$listId";
	if(!($fields = clearFusionCMS::getCache()->get($cacheId)) && !is_array($fields)) {
		if(!is_array($fields = $mc->getVariables($listId)))
			$fields = '-';
		clearFusionCMS::getCache()->store($cacheId, $fields, is_array($fields) ? 86400 : 3600);
	}

	if(is_array($fields)) {
		if(isset($fldOrder)) {
			$fldOrder = preg_split('#\s*,\s*#', $fldOrder);
			$flds = array();
			foreach($fldOrder as $o) {
				foreach($fields as $f) {
					if($f->tag == $o) {
						$flds[$o] = $f;
						break;
					}
				}
			}

			foreach($fields as $f) {
				if(!isset($flds[$f->tag]))
					$flds[$f->tag] = $f;
			}

			$fields = array_values($flds);
		}

		$prefix = isset($prefix) ? $prefix : '';
		$form = new flForm(clearFusionCMS::getRequest(), $prefix . 'mailchimpsubscribe', clearFusionCMS::getRequest()->getSelf());

		foreach($fields as $f) {
			if($f->public) {
				switch($f->field_type) {
					case 'email':
						$elem = new flFormElementEmail($form, $prefix . $f->tag, isset($email) ? $email : null);
						break;
					case 'number':
						$elem = new flFormElementNumber($form, $prefix . $f->tag);
						break;
					case 'radio':
						$d = array();
						foreach($f->choices as $v)
							$d[$v] = $v;
						$elem = new flFormElementRadio($form, $prefix . $f->tag, '', $d);
						break;
					case 'dropdown':
						$d = array();
						foreach($f->choices as $v)
							$d[$v] = $v;
						$elem = new flFormElementSelect($form, $prefix . $f->tag, '', $d);
						break;
					case 'url':
						$elem = new flFormElementUri($form, $prefix . $f->tag);
						break;
					default:
						$elem = new flFormElementText($form, $prefix . $f->tag);
						break;
				}

				$elem->setLabel($f->name);
				if($f->req)
					$elem->setRequired()->addValidator(new flFormValidatorRequired());
			}
		}

		$submit = new flFormElementSubmit($form, 'mcsubscribe');
		$submit->setCaption($caption)->setCSS('primary');

		$showForm = true;
		if($form->isSubmitted() && $form->isValid()) {
			$d = array();
			$e = '';
			foreach($fields as $f) {
				if($f->public) {
					if($f->tag == 'EMAIL')
						$e = $form->getElement($prefix . $f->tag)->getValue();
					else
						$d[$f->tag] = $form->getElement($prefix . $f->tag)->getValue();
				}
			}

			if($mc->subscribe($listId, $e, $d)) {
				if(isset($redirectTo))
					clearFusionCMS::getResponse()->redirectTo($controller->content->getPageURI($redirectTo, true));
				else {
					$showForm = false;
					echo isset($subscribedMsg) ? $subscribedMsg : __("<p>You have been subscribed to our mailing list.</p>", 'mailchimp');
				}
			}
			else
				echo $form->getElement('EMAIL')->forceError(__('We are currently unable to subscribe you to the mailing list.', 'mailchimp'));
		}

		if($showForm)
			echo $form->render($tplEngine->getCodeBuffer());
	}
}
