<?php

/**
 * Snippet to display MailChimp update form.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\mailchimp
 * @copyright Copyright (c) 2013 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/* @var $tplEngine cfTemplateEngine */
$controller = $tplEngine->controller;

if(isset($listId) && isset($email)) {
	$mc = new mailchimp();

	// Get the users current information
	if(!($subscriber = $mc->getSubscriber($listId, $email)) || !$subscriber->success) {
		echo isset($notSubscribedMsg) ? $notSubscribedMsg : __('<p>You are not subscribed to this mailing list.</p>', 'mailchimp');
		return;
	}

	// Cache field information
	$cacheId = '__cfMailChimp' . clearFusionCMS::getDb()->getPrefix() . "-fields-$listId";
	if(!($fields = clearFusionCMS::getCache()->get($cacheId)) && !is_array($fields)) {
		if(!is_array($fields = $mc->getVariables($listId)))
			$fields = '-';
		clearFusionCMS::getCache()->store($cacheId, $fields, is_array($fields) ? 86400 : 3600);
	}

	if(is_array($fields)) {
		$form = new flForm(clearFusionCMS::getRequest(), 'mailchimpupdate', clearFusionCMS::getRequest()->getSelf());

		$data = $subscriber->data[0]->merges;
		foreach($fields as $f) {
			$name = $f->tag;
			switch($f->field_type) {
				case 'email': $elem = new flFormElementEmail($form, $name, $data->$name); break;
				case 'number': $elem = new flFormElementNumber($form, $name, null, null, $data->$name); break;
				case 'radio':
					$d = array();
					foreach($f->choices as $v)
						$d[$v] = $v;
					$elem = new flFormElementRadio($form, $name, $data->$name, $d);
					break;
				case 'dropdown':
					$d = array();
					foreach($f->choices as $v)
						$d[$v] = $v;
					$elem = new flFormElementSelect($form, $name, $data->$name, $d);
					break;
				case 'url': $elem = new flFormElementUri($form, $name, $data->$name); break;
				default: $elem = new flFormElementText($form, $name, $data->$name); break;
			}

			$elem->setLabel($f->name);
			if($f->req)
				$elem->setRequired()->addValidator(new flFormValidatorRequired());
		}

		$submit = new flFormElementSubmit($form, 'mcupdate');
		$submit->setCaption(__('Update', 'mailchimp'))->setCSS('primary');

		$showForm = true;
		if($form->isSubmitted() && $form->isValid()) {
			$d = array();
			$subEmail = '';
			foreach($fields as $f) {
				if($f->tag == 'EMAIL')
					$subEmail = $form->getElement($f->tag)->getValue();
				else
					$d[$f->tag] = $form->getElement($f->tag)->getValue();
			}

			if($mc->update($listId, $email, $subEmail, $d)) {
				$showForm = false;
				echo isset($updateMsg) ? $updateMsg : __("<p>Your details have been updated.</p>", 'mailchimp');
			}
			else
				echo $form->getElement('EMAIL')->forceError(__('We are currently unable to update your subscription.', 'mailchimp'));
		}

		if($showForm)
			echo $form->render($tplEngine->getCodeBuffer());
	}
}
