<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\markdown
 * @copyright Copyright (c) 2016 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * markdownWYSIWYG Class
 *
 * Class to draw the editor.
 *
 * @package clearFusionCMS\module\markdown
 */
class markdownWYSIWYG extends codemirrorWYSIWYG {

	/**
	 * @inheritdoc
	 */
	function __construct($form, $name) {
		parent::__construct($form, $name);

		$request = $form->getRequest();

		if($request->isXmlHttpRequest() && $request->isPost() && $request->hasPost('__markdown') && $request->getPost('name') == $this->name) {

			require_once FLAPPPATH . 'parsedown/parsedown.php';
			$parsedown = new Parsedown();

			// Send AJAX response
			$response = new flResponse($request);
			$response->setContentType('text/plain')
				->noCache()
				->setBody($parsedown->text($request->getPost('__markdown')))
				->sendResponse();

			// Stop processing the script
			die;
		}
	}

	/**
	 * @inheritdoc
	 */
	function preRender($db, $codeBuffer, $settings) {
		return '<div id="' . $this->name . '_markdownEditor"><div class="row"><div class="span-s6">';
	}

	/**
	 * @inheritdoc
	 */
	function postRender($db, $codeBuffer, $settings) {

		$w = $settings['width'];
		$h = $settings['height'];

		if(!strpos($w, '%') !== false)
			$w .= 'px';
		if(!strpos($h, '%') !== false)
			$h .= 'px';

		return '</div><div class="span-s6"><div class="markdownPreview" id="' . $this->name . '_preview" style="width: ' . $w . '; height: ' . $h . ';"></div></div></div></div>';
	}

	/**
	 * @inheritdoc
	 */
	function render($db, $codeBuffer, $settings) {
		$codeBuffer->bufferCSS('system/modules/markdown/editor.css')
			->bufferCSS('system/app/prism/prism.css')
			->bufferBodyJS('system/app/prism/prism.js')
			->requireUtils();

		// Use our config and disable codemirror fullscreen mode
		$settings['_internal_cfg'] = clearFusionCMS::getSiteConfig('cfMarkdown');
		$settings['allowFullScreen'] = false;

		// Capture escape key presses and toggle full screen
		$settings['_internal_opts'] = 'extraKeys: {
			"Esc": function(cm) {
				var e = $("#' . $this->name . '_markdownEditor");
				
				if(e.hasClass("markdownEditor-fullscreen")) {
					$("body").removeClass("markdownEditorBodyLock");
					e.removeClass("markdownEditor-fullscreen");
				}
				else {
					$("body").addClass("markdownEditorBodyLock");
					e.addClass("markdownEditor-fullscreen");
				}
				
				cm.refresh();
			}
		},';

		// Add additional events
		$settings['_internal_events'] = array(
			// Capture change events and update the preview
			'change' => 'queueUpdatePreview',
			'scroll' => 'cmScroll'
		);

		$edName = str_replace('.', '_', $this->name) . '_CodeMirror';

		$settings['_internal_js'] = "function updatePreview() {
			// ajax convert the markdown to 
			var preview = \$('#{$this->name}_preview');
			var form = preview.closest('form');
			var action = form.attr('action');
			
			\$.ajax(action, {
				type: 'POST',
				timeout: 3000,
				success: function(d) {
					\$('#{$this->name}_preview').html(d);
					Prism.highlightAll();
				},
				data: {
					name: '$this->name',
					__markdown: $edName.getValue()
				}
			});
		}
		function queueUpdatePreview() {
			var preview = \$('#{$this->name}_preview');
			preview.timerStop('markdownPreview', updatePreview)
				.timerOnce(500, 'markdownPreview', updatePreview);
		}
		function cmScroll(cm) {
			var p = cm.getScrollInfo();
			\$('#{$this->name}_preview').scrollTop(p.top);
		}
		\$(document).ready(function() {
			updatePreview();
			
			setTimeout(function() {
				var p = $edName.getScrollInfo();
				\$('#{$this->name}_preview').scrollTop(p.top);
			}, 200);
		});
		";

		return parent::render($db, $codeBuffer, $settings);
	}
}
