<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\slack
 * @copyright Copyright (c) 2016 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * slackEvents Class
 *
 * Class to process Slack module events.
 *
 * @package clearFusionCMS\module\slack
 */
class slackEvents {

	/**
	 * Event handler for system notify updates.
	 *
	 * @param flEvent $evt The event object.
	 */
	function onNotify($evt) {
		$cfg = clearFusionCMS::getSiteConfig('cfSlack');

		if(!empty($cfg->hookURL) && ($level = $evt->getParam('level', 1)) >= $cfg->level) {
			$colorMap = array(1 => '#9E9E9E', 2 => '#9C27B0', 3 => '#4CAF50', 4 => '#FFEB3B', 5 => '#F44336' );
			$level = min(max($level, 1), 5);

			$title = $evt->getParam('title', __('Website Notification', 'slackBackend'));
			$link = $evt->getParam('link', '');
			$message = $evt->getParam('message', '');
			$mention = trim($evt->getParam('mention', ''));
			$fields = $evt->getParam('fields', array());

			if(!empty($mention))
				$message = $mention . ' ' . $message;

			$msg = array(
				'fallback' => $message,
				'color' => $colorMap[$level],
				'title' => $title,
				'text' => $message,
				'footer' => clearFusionCMS::getSiteName(),
				'ts' => time()
			);

			if(!empty($link))
				$msg['title_link'] = $link;

			if(count($fields)) {
				$msg['fields'] = array();
				foreach($fields as $v) {
					$msg['fields'][] = array(
						'title' => $v['title'],
						'value' => $v['value'],
						'short' => false
					);
				}
			}

			$client = new flHttpClient($cfg->hookURL);
			$client->setPostData(
				array(
					'payload' => json_encode(
						array(
							'attachments' => array(
								$msg
							)
						)

					)
				)
			);

			$client->sendRequest('POST');
		}
	}
}
