<?php

/**
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\slack
 * @copyright Copyright (c) 2016 - 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * slackSettings Class
 *
 * Class to implement settings for the Slack module.
 *
 * @package clearFusionCMS\module\slack
 */
class slackSettings implements cfModuleSettingsInterface {

	/**
	 * Get a list of settings groups.
	 *
	 * @return array Array of setting groups, name => func name
	 */
	function getSettingsGroups() {
		if(!clearFusionCMS::getUser()->hasPermission('cfSlack.settings'))
			return array();

		return array(
			__('Settings', 'slackBackend') => 'general'
		);
	}

	/**
	 * Implement Slack settings.
	 *
	 * @param flForm $form The form object to use.
	 * @param flDb $db The database object.
	 * @return bool true if the settings were updated, false if failed to update or null if no change.
	 */
	function generalSetting($form, $db) {
		if(!clearFusionCMS::getUser()->hasPermission('cfSlack.settings'))
			return null;

		$cfg = clearFusionCMS::getSiteConfig('cfSlack');

		$webhook = new flFormElementUri($form, 'webhook', $cfg->hookURL);
		$webhook->setLabel(__('Webhook URL', 'slackBackend'))
			->setDescription(__('The URL of the Slack Webhook.', 'slackBackend'))
			->setRequired()
			->addValidator(new flFormValidatorRequired());

		$level = new flFormElementSelect($form, 'level', $cfg->level, array(100 => __('Disabled', 'slackBackend'), 1 => '1', 2 => '2', 3 => '3', 4 => '4', 5 => '5'));
		$level->setLabel(__('Notification Level', 'slackBackend'))
			->setDescription(__('Notifications below the selected level will be ignored.', 'slackBackend'));

		$submit = new flFormElementSubmit($form, 'update');
		$submit->setCaption(__('Update Settings', 'slackBackend'))->setCSS('primary');

		$test = new flFormElementSubmit($form, 'test');
		$test->setCaption(__('Update &amp; Test Settings', 'slackBackend'))->setCSS('accent');

		$form->groupElements('submit', array($submit, $test), new flFormDecoratorList(new flFormDecoratorDl()));
		$form->focusElement($webhook);

		if($form->isSubmitted() && $form->isValid()) {
			cfSettingsAudit::update(
				__('Slack', 'slackBackend'),
				__('Settings', 'slackBackend'),
				[
					'hookURL' => $cfg->get('hookURL'),
					'level' => $cfg->get('level')
				]
			);

			$cfg->hookURL = $webhook->getValue();
			$cfg->level = $level->getValue();

			// Send a test message
			if($test->getValue()) {
				$sink = new slackEvents();
				$sink->onNotify(new cfEventNotify(
					$level->getValue(),
					__('Slack Configuration Test', 'slackBackend'),
					sprintf(__('%s has configured the Slack module, the integration works!', 'slackBackend'), clearFusionCMS::getUser()->getUsername())
				));
			}

			return true;
		}

		return null;
	}
}
