<?php

/**
 * Class file for decoding social module audit logs.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\social
 * @copyright Copyright (c) 2017 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/**
 * socialAuditHelper Class
 *
 * Class to provide decoding for social audit trails.
 *
 * @package clearFusionCMS\module\social
 */
class socialAuditHelper extends cfAuditHelper {

	/**
	 * Convert the raw data to a human readable message.
	 *
	 * @param array $data The audit data.
	 * @return string The message string.
	 */
	function getMessage($data) {
		$params = empty($data['params']) ? [] : json_decode($data['params'], true);

		$title = isset($params['title']) ? $params['title'] : '';

		$link = '';
		if($info = clearFusionCMS::getDb()->fetchRow('SELECT feedid FROM {cf_socialtwitter} WHERE feedid=?', $data['cid'])) {
			$link = ' <a href="' . clearFusionCMS::getController()->assemble('social', 'twitter', ['edit', $data['cid']]) . '">' . __('Edit the Feed', 'blogBackend') . '</a>';
		}

		switch($data['action']) {
			case socialAudit::CREATE:
				$msg = sprintf(__('Added Twitter feed <strong>%s</strong>.', 'socialBackend'), $title) . $link;
				break;
			case socialAudit::UPDATE:
				$msg = sprintf(__('Modified the Twitter feed <strong>%s</strong>.', 'socialBackend'), $title) . $link;
				break;
			case socialAudit::DELETE:
				$msg = sprintf(__('Deleted the Twitter feed <strong>%s</strong>.', 'socialBackend'), $title);
				break;
			default:
				$msg = parent::getMessage($data);
				break;
		}

		return $msg;
	}

	/**
	 * @inheritdoc
	 */
	function hasAdditionalData($data) {
		return !in_array($data['action'], [ socialAudit::CREATE ]);
	}
}
