<?php

/**
 * Snippet to display TOLRA Mail subscribe form.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package clearFusionCMS\module\tolramail
 * @copyright Copyright (c) 2013 clearFusionCMS. All rights reserved.
 * @link http://clearfusioncms.com
 */

/* @var $tplEngine cfTemplateEngine */
$controller = $tplEngine->controller;

if(isset($listId)) {
	$cm = new tolramail();

	// Cache field information
	$cacheId = '__cfTOLRAMail' . clearFusionCMS::getDb()->getPrefix() . "-fields-$listId";
	if(!($fields = clearFusionCMS::getCache()->get($cacheId)) && !is_array($fields)) {
		if(!is_array($fields = $cm->getFields($listId)))
			$fields = '-';
		clearFusionCMS::getCache()->store($cacheId, $fields, is_array($fields) ? 86400 : 3600);
	}

	if(!is_array($fields)) {
		echo __("<p>Newsletter subscriptions are currently unavailable please try again later.</p>", 'tolramail');
		return;
	}

	$prefix = isset($prefix) ? $prefix : '';

	$form = new flForm(clearFusionCMS::getRequest(), $prefix . 'tolramailsubscribe', clearFusionCMS::getRequest()->getSelf());

	$e = new flFormElementEmail($form, $prefix . 'cmemail', isset($email) ? $email : null);
	$e->setLabel(__('Email Address', 'tolramail'))
		->setRequired()
		->addValidator(new flFormValidatorRequired());

	$name = new flFormElementText($form, $prefix . 'cmname');
	$name->setLabel(__('Your Name', 'tolramail'));

	foreach($fields as $f) {
		switch($f->DataType) {
			case 'Number': $elem = new flFormElementNumber($form, md5($prefix . $f->Key)); break;
			case 'MultiSelectOne':
				$d = array();
				foreach($f->FieldOptions as $v)
					$d[$v] = $v;
				$elem = new flFormElementSelect($form, md5($prefix . $f->Key), '', $d);
				break;
			case 'MultiSelectMany':
				$d = array();
				foreach($f->FieldOptions as $v)
					$d[$v] = $v;
				$elem = new flFormElementChecklist($form, md5($prefix . $f->Key), array(), $d);
				break;
			default: $elem = new flFormElementText($form, md5($prefix . $f->Key)); break;
		}

		$elem->setLabel($f->FieldName);
	}

	$submit = new flFormElementSubmit($form, 'cmsubscribe');
	$submit->setCaption(__('Subscribe', 'tolramail'))->setCSS('primary');

	$showForm = true;
	if($form->isSubmitted() && $form->isValid()) {
		$d = array();
		foreach($fields as $f) {
			$v = $form->getElement(md5($prefix . $f->Key))->getValue();
			if(is_array($v)) {
				foreach($v as $sel)
					$d[] = array('Key' => $f->Key, 'Value' => $sel);
			}
			else
				$d[] = array('Key' => $f->Key, 'Value' => $v);
		}

		if($cm->subscribe($listId, $e->getValue(), $name->getValue(), $d)) {
			if(isset($redirectTo))
				clearFusionCMS::getResponse()->redirectTo($controller->content->getPageURI($redirectTo, true));
			else {
				$showForm = false;
				echo isset($subscribedMsg) ? $subscribedMsg : __("<p>You have been subscribed to our mailing list.</p>", 'tolramail');
			}
		}
		else
			echo $e->forceError(__('We are currently unable to subscribe you to the mailing list.', 'tolramail'));
	}

	if($showForm)
		echo $form->render($tplEngine->getCodeBuffer());
}
